@@ -8,6 +8,7 @@ import base64
 import onnxruntime
 from PIL import Image
 import numpy as np
+import cv2
 
 
 def base64_to_image(img_base64):
@@ -21,9 +22,20 @@ def get_img_base64(single_image_path):
         return img_base64.decode()
 
 
+class TypeError(Exception):
+    pass
+
+
 class DdddOcr(object):
-    def __init__(self, use_gpu: bool = False, device_id: int = 0):
-        self.__graph_path = os.path.join(os.path.dirname(__file__), 'common.onnx')
+    def __init__(self, det: bool = False, use_gpu: bool = False, device_id: int = 0, show_ad=True):
+        print("欢迎使用ddddocr，本项目专注带动行业内卷，个人博客:wenanzhe.com")
+        if show_ad:
+            print("训练数据支持来源于:http://146.56.204.113:19199/preview")
+        if det:
+            self.__graph_path = os.path.join(os.path.dirname(__file__), 'common_det.onnx')
+        else:
+            self.__graph_path = os.path.join(os.path.dirname(__file__), 'common.onnx')
+        self.det = det
         if use_gpu:
             self.__providers = [
                 ('CUDAExecutionProvider', {
@@ -554,7 +566,144 @@ class DdddOcr(object):
                           "举", "瞓", "谬", "溝", "言", "哽", "婿", "猿", "跗", "獴", "俜", "呙", "弗", "凿", "窭", "铌",
                           "友", "唉", "怫", "荘"]
 
+    def preproc(self, img, input_size, swap=(2, 0, 1)):
+        if len(img.shape) == 3:
+            padded_img = np.ones((input_size[0], input_size[1], 3), dtype=np.uint8) * 114
+        else:
+            padded_img = np.ones(input_size, dtype=np.uint8) * 114
+
+        r = min(input_size[0] / img.shape[0], input_size[1] / img.shape[1])
+        resized_img = cv2.resize(
+            img,
+            (int(img.shape[1] * r), int(img.shape[0] * r)),
+            interpolation=cv2.INTER_LINEAR,
+        ).astype(np.uint8)
+        padded_img[: int(img.shape[0] * r), : int(img.shape[1] * r)] = resized_img
+
+        padded_img = padded_img.transpose(swap)
+        padded_img = np.ascontiguousarray(padded_img, dtype=np.float32)
+        return padded_img, r
+
+    def demo_postprocess(self, outputs, img_size, p6=False):
+        grids = []
+        expanded_strides = []
+
+        if not p6:
+            strides = [8, 16, 32]
+        else:
+            strides = [8, 16, 32, 64]
+
+        hsizes = [img_size[0] // stride for stride in strides]
+        wsizes = [img_size[1] // stride for stride in strides]
+
+        for hsize, wsize, stride in zip(hsizes, wsizes, strides):
+            xv, yv = np.meshgrid(np.arange(wsize), np.arange(hsize))
+            grid = np.stack((xv, yv), 2).reshape(1, -1, 2)
+            grids.append(grid)
+            shape = grid.shape[:2]
+            expanded_strides.append(np.full((*shape, 1), stride))
+
+        grids = np.concatenate(grids, 1)
+        expanded_strides = np.concatenate(expanded_strides, 1)
+        outputs[..., :2] = (outputs[..., :2] + grids) * expanded_strides
+        outputs[..., 2:4] = np.exp(outputs[..., 2:4]) * expanded_strides
+
+        return outputs
+
+    def nms(self, boxes, scores, nms_thr):
+        """Single class NMS implemented in Numpy."""
+        x1 = boxes[:, 0]
+        y1 = boxes[:, 1]
+        x2 = boxes[:, 2]
+        y2 = boxes[:, 3]
+
+        areas = (x2 - x1 + 1) * (y2 - y1 + 1)
+        order = scores.argsort()[::-1]
+
+        keep = []
+        while order.size > 0:
+            i = order[0]
+            keep.append(i)
+            xx1 = np.maximum(x1[i], x1[order[1:]])
+            yy1 = np.maximum(y1[i], y1[order[1:]])
+            xx2 = np.minimum(x2[i], x2[order[1:]])
+            yy2 = np.minimum(y2[i], y2[order[1:]])
+
+            w = np.maximum(0.0, xx2 - xx1 + 1)
+            h = np.maximum(0.0, yy2 - yy1 + 1)
+            inter = w * h
+            ovr = inter / (areas[i] + areas[order[1:]] - inter)
+
+            inds = np.where(ovr <= nms_thr)[0]
+            order = order[inds + 1]
+
+        return keep
+
+    def multiclass_nms_class_agnostic(self, boxes, scores, nms_thr, score_thr):
+        """Multiclass NMS implemented in Numpy. Class-agnostic version."""
+        cls_inds = scores.argmax(1)
+        cls_scores = scores[np.arange(len(cls_inds)), cls_inds]
+
+        valid_score_mask = cls_scores > score_thr
+        if valid_score_mask.sum() == 0:
+            return None
+        valid_scores = cls_scores[valid_score_mask]
+        valid_boxes = boxes[valid_score_mask]
+        valid_cls_inds = cls_inds[valid_score_mask]
+        keep = self.nms(valid_boxes, valid_scores, nms_thr)
+        if keep:
+            dets = np.concatenate(
+                [valid_boxes[keep], valid_scores[keep, None], valid_cls_inds[keep, None]], 1
+            )
+        return dets
+
+    def multiclass_nms(self, boxes, scores, nms_thr, score_thr):
+        """Multiclass NMS implemented in Numpy"""
+        return self.multiclass_nms_class_agnostic(boxes, scores, nms_thr, score_thr)
+
+    def get_bbox(self, image_bytes):
+        img = cv2.imdecode(np.frombuffer(image_bytes, np.uint8), cv2.IMREAD_COLOR)
+
+        im, ratio = self.preproc(img, (416, 416))
+        ort_inputs = {self.__ort_session.get_inputs()[0].name: im[None, :, :, :]}
+        output = self.__ort_session.run(None, ort_inputs)
+        predictions = self.demo_postprocess(output[0], (416, 416))[0]
+        boxes = predictions[:, :4]
+        scores = predictions[:, 4:5] * predictions[:, 5:]
+
+        boxes_xyxy = np.ones_like(boxes)
+        boxes_xyxy[:, 0] = boxes[:, 0] - boxes[:, 2] / 2.
+        boxes_xyxy[:, 1] = boxes[:, 1] - boxes[:, 3] / 2.
+        boxes_xyxy[:, 2] = boxes[:, 0] + boxes[:, 2] / 2.
+        boxes_xyxy[:, 3] = boxes[:, 1] + boxes[:, 3] / 2.
+        boxes_xyxy /= ratio
+
+        pred = self.multiclass_nms(boxes_xyxy, scores, nms_thr=0.45, score_thr=0.1)
+        final_boxes = pred[:, :4].tolist()
+        result = []
+        for b in final_boxes:
+            if b[0] < 0:
+                x_min = 0
+            else:
+                x_min = int(b[0])
+            if b[1] < 0:
+                y_min = 0
+            else:
+                y_min = int(b[1])
+            if b[2] > img.shape[1]:
+                x_max = int(img.shape[1])
+            else:
+                x_max = int(b[2])
+            if b[3] > img.shape[0]:
+                y_max = int(img.shape[0])
+            else:
+                y_max = int(b[3])
+            result.append([x_min, y_min, x_max, y_max])
+        return result
+
     def classification(self, img_bytes: bytes = None, img_base64: str = None):
+        if self.det:
+            raise TypeError("当前识别类型为目标检测")
         if img_bytes:
             image = Image.open(io.BytesIO(img_bytes))
         else:
@@ -576,3 +725,12 @@ class DdddOcr(object):
                 result.append(self.__charset[item])
 
         return ''.join(result)
+
+    def detection(self, img_bytes: bytes = None, img_base64: str = None):
+        if not self.det:
+            raise TypeError("当前识别类型为文字识别")
+        if not img_bytes:
+            img_bytes = base64.b64decode(img_base64)
+        result = self.get_bbox(img_bytes)
+
+        return result
